<?php

namespace App\Http\Controllers;

use App\Models\Check;
use App\Models\Transaction;
use App\Models\Attachment;
use App\Services\CheckeeperService;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Mail;
use App\Mail\CheckDetailsMail;

class CheckController extends Controller
{
    protected $checkeeperService;

    /**
     * Constructor to inject CheckeeperService
     *
     * @param CheckeeperService $checkeeperService
     */
    public function __construct(CheckeeperService $checkeeperService)
    {
        $this->checkeeperService = $checkeeperService;
    }

    /**
     * Display a listing of the checks.
     */
    public function index()
    {
        // Fetch all checks with pagination (adjust as needed)
        $checks = Check::paginate(10);

        return view('admin.checks.index', compact('checks'));
    }

    /**
     * Storing new data from Cross-checked data for Checkeeper to checks table
     *
     * @param int $checkId
     * @return \Illuminate\Http\JsonResponse
     */
public function store(Request $request)
    {
        // Validate the incoming request
        $validated = $request->validate([
            'nonce' => 'required|string',
            'date' => 'required|date',
            'amount' => 'required|numeric',
            'number' => 'required|string',
            'memo' => 'nullable|string',
            'payee_line1' => 'nullable|string',
            'bank_routing' => 'required|string',
            'bank_account' => 'required|string',
            'payer_line1' => 'nullable|string',
            'payer_line2' => 'nullable|string',
            'payer_line3' => 'nullable|string',
            'payer_email' => 'nullable|string',
            'signer_type' => 'nullable|string',
            'signer_value' => 'nullable|string',
            'signer_value2' => 'nullable|string',
            'delivery_method' => 'required|string',

            // To Address Fields
            'to_address_name' => 'required|string',
            'to_address_line1' => 'required|string',
            'to_address_city' => 'required|string',
            'to_address_state' => 'required|string',
            'to_address_zip' => 'required|string',
            'to_address_country' => 'required|string',

            // From Address Fields
            'from_address_name' => 'nullable|string',
            'from_address_line1' => 'nullable|string',
            'from_address_line2' => 'nullable|string',
             'from_address_city' => 'nullable|string',
              'from_address_state' => 'nullable|string',
               'from_address_zip' => 'nullable|string',
                'from_address_country' => 'nullable|string',
        ]);

        // Start a database transaction
        DB::beginTransaction();

        try {
            // Store the check in the checks table
            $check = Check::create([
                'nonce' => $validated['nonce'],
                'date' => $validated['date'],
                'amount' => $validated['amount'],
                'number' => $validated['number'],
                'memo' => $validated['memo'],
                'payee' => [
                    'line1' => $validated['payee_line1'],
                ],
                'bank' => [
                    'routing' => $validated['bank_routing'],
                    'account' => $validated['bank_account'],
                ],
                'payer' => [
                    'line1' => $validated['payer_line1'],
                    'line2' => $validated['payer_line2'],
                    'line3' => $validated['payer_line3'],
                    'email' => $validated['payer_email'],
                ],
                'signer' => [
                    'type' => $validated['signer_type'],
                    'value' => $validated['signer_value'],
                    'value2' => $validated['signer_value2'],
                ],
                'to_address' => [
                    'name' => $validated['to_address_name'],
                    'line1' => $validated['to_address_line1'],
                    'city' => $validated['to_address_city'],
                    'state' => $validated['to_address_state'],
                    'zip' => $validated['to_address_zip'],
                    'country' => $validated['to_address_country'],
                ],
                'from_address' => [
                    'name' => $validated['from_address_name'],
                    'line1' => $validated['from_address_line1'],
                    'line2' => $validated['from_address_line2'],
                    'city' => $validated['from_address_city'],
                    'state' => $validated['from_address_state'],
                    'zip' => $validated['from_address_zip'],
                    'country' => $validated['from_address_country'],
                ],
            ]);

            // Store the delivery information in the transactions table
            $transaction = Transaction::create([
                'check_id' => $check->id, // Associate transaction with the check
                'delivery' => [
                    'method' => $validated['delivery_method'],
                ],
            ]);

            // Update the `transaction_id` in the check record
            $check->transaction_id = $transaction->id;
            $check->save();

            // Commit the transaction
            DB::commit();

            return response()->json([
                'message' => 'Check and transaction stored successfully.',
                'check' => $check,
                'transaction' => $transaction,
            ], 201);
        } catch (\Exception $e) {
            // Rollback the transaction in case of an error
            DB::rollBack();

            // Log the error for debugging
            Log::error('Failed to store check and transaction', [
                'error' => $e->getMessage(),
                'trace' => $e->getTraceAsString(),
            ]);

            return response()->json([
                'error' => 'Failed to store check and transaction',
                'details' => $e->getMessage(),
            ], 500);
        }
    }


    
  public function createCheck($checkId)
{
    // Fetch the check by ID
    $check = Check::find($checkId);

    if (!$check) {
        return response()->json([
            'error' => 'Check not found',
            'details' => 'No check exists with the provided ID.',
        ], 404);
    }

    // Fetch related transaction data
    $transaction = Transaction::where('check_id', $checkId)->first();

    if (!$transaction) {
        return response()->json([
            'error' => 'Transaction not found',
            'details' => 'No transaction exists for the provided check ID.',
        ], 404);
    }

    // Retrieve the Attachment
    $attachment = Attachment::first(); // Assumes only one attachment exists

    if (!$attachment) {
        return response()->json([
            'error' => 'Attachment not found',
            'details' => 'No attachment has been uploaded. Please upload an attachment before creating a check.',
        ], 400);
    }

    // Construct the Payload with Attachment
    $payload = [
        'checks' => [
            [
                'nonce' => Str::uuid()->toString(),
                'date' => $check->date,
                'amount' => (int) ($check->amount * 100),
                'number' => $check->number,
                'memo' => $check->memo,
                'payee' => [
                    'line1' => $check->payee['line1'],
                ],
                'bank' => [
                    'routing' => str_repeat('*', strlen($check->bank['routing']) - 4) . substr($check->bank['routing'], -4),
                    'account' => str_repeat('*', strlen($check->bank['account']) - 4) . substr($check->bank['account'], -4),
                ],
                'payer' => [
                    'line1' => $check->payer['line1'],
                    'line2' => $check->payer['line2'] ?? '',
                    'line3' => $check->payer['line3'] ?? '',
                    'email' => $check->payer['email'] ?? '',
                ],
                'signer' => [
                    'type' => $check->signer['type'] ?? 'default',
                    'value' => $check->signer['value'] ?? '',
                    'value2' => $check->signer['value2'] ?? '',
                ],
                'to_address' => [
                    'name' => $check->to_address['name'] ?? 'Default Name',
                    'line1' => $check->to_address['line1'] ?? 'Default Line1',
                    'city' => $check->to_address['city'] ?? 'Default City',
                    'state' => $check->to_address['state'] ?? 'Default State',
                    'zip' => $check->to_address['zip'] ?? '00000',
                    'country' => $check->to_address['country'] ?? 'US',
                ],
                'from_address' => [
                    'name' => $check->from_address['name'] ?? 'Default Name',
                    'line1' => $check->from_address['line1'] ?? 'Default Line1',
                    'line2' => $check->from_address['line2'] ?? '',
                    'line3' => $check->from_address['line3'] ?? '',
                    'city' => $check->from_address['city'] ?? 'Default City',
                    'state' => $check->from_address['state'] ?? 'Default State',
                    'zip' => $check->from_address['zip'] ?? '00000',
                    'country' => $check->from_address['country'] ?? 'US',
                ],
                'attachments' => [
                    $attachment->base64_content
                ],
                'template' => 'default'
            ],
        ],
        'delivery' => [
            'method' => $transaction->delivery['method'],
        ],
    ];

    Log::info('Checkeeper Payload Sent:', $payload);

    // Send the request to Checkeeper API using the service
    try {
        $response = $this->checkeeperService->sendChecks($payload);
    } catch (\Exception $e) {
        Log::error('API Request Failed for check ID ' . $checkId . ': ' . $e->getMessage());
        return response()->json([
            'error' => 'API Request Failed',
            'details' => $e->getMessage(),
        ], 500);
    }

    // Handle the API response
    if ($response->successful()) {
        $apiResponse = $response->json();

        // Capture and store the PDF Base64 string
        if (isset($apiResponse['data']['pdf'])) {
        // Decode the Base64 string
        $pdfContent = base64_decode($apiResponse['data']['pdf']);
        
        // Define the file name and path
        $fileName = uniqid('check_') . '.pdf';
        $filePath = 'payment/checkPdf/' . $fileName;
        
        // Save the PDF file to the disk
        Storage::disk('checkPdf')->put($fileName, $pdfContent);
    
        // Store the file path in the database (so you can retrieve it later)
        $check->pdf_url = $filePath;
        $check->save();
    }


        // Add a success message to the session
        session()->flash('success', 'Check created successfully!');

        // Return the view with the alert
        $checks = Check::paginate(10);
        return view('admin.checks.index', compact('checks'));
    } else {
        $errorDetails = $response->json();

        if (json_last_error() !== JSON_ERROR_NONE) {
            $errorDetails = $response->body();
        }

        Log::error('Failed to create check in Checkeeper API for check ID ' . $checkId . ': ' . $response->body());
        return response()->json([
            'error' => 'Failed to create check',
            'details' => $errorDetails,
        ], 500);
    }
}

public function sendMail($id)
{
    $check = Check::findOrFail($id);

    // Define the path to the attachment
    $attachmentPath = base_path('/public/attachment.pdf');

    if (!file_exists($attachmentPath)) {
        Log::error("Attachment not found: {$attachmentPath}");
        return redirect()->back()->with('error', 'Attachment file not found.');
    }

    try {
        // Send the email
        Mail::to($check->payer['email'])->send(new CheckDetailsMail($check, $attachmentPath));

        // Log the success
        Log::info("Email sent successfully to {$check->payer['email']} for Check ID: {$check->id}");
        return redirect()->back()->with('success', 'Email sent successfully.');

    } catch (\Exception $e) {
        // Log the error
        Log::error("Failed to send email to {$check->payer['email']} for Check ID: {$check->id}. Error: {$e->getMessage()}");
        return redirect()->back()->with('error', 'Failed to send the email. Please try again.');
    }
}



}
