<?php

namespace App\Http\Controllers;

use App\Models\Transaction;
use App\Models\Check;
use App\Services\CheckeeperService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;

class TransactionController extends Controller
{
    protected $checkeeperService;

    /**
     * Constructor to inject CheckeeperService
     *
     * @param CheckeeperService $checkeeperService
     */
    public function __construct(CheckeeperService $checkeeperService)
    {
        $this->checkeeperService = $checkeeperService;
    }

    /**
     * Store a newly created transaction with checks.
     */
    public function store(Request $request)
    {
        // Validate the incoming request data
        $validator = Validator::make($request->all(), [
            // Validation rules for delivery
            'delivery.method' => 'required|string|in:pdf,mail',

            // Validation rules for checks
            'checks' => 'required|array|min:1',
            'checks.*.nonce' => 'required|string|max:255|unique:checks,nonce',
            'checks.*.date' => 'required|date',
            'checks.*.amount' => 'required|integer|min:0',
            'checks.*.number' => 'required|string|max:50',
            'checks.*.memo' => 'nullable|string|max:255',

            // Validation for nested payee object
            'checks.*.payee' => 'required|array',
            'checks.*.payee.line1' => 'required|string|max:255',

            // Validation for nested bank object
            'checks.*.bank' => 'required|array',
            'checks.*.bank.routing' => 'required|string|digits:9',
            'checks.*.bank.account' => 'required|string|digits_between:6,17',

            // Validation for nested payer object
            'checks.*.payer' => 'required|array',
            'checks.*.payer.line1' => 'required|string|max:255',
            'checks.*.payer.line2' => 'nullable|string|max:255',
            'checks.*.payer.line3' => 'nullable|string|max:255',

            // Validation for nested signer object
            'checks.*.signer' => 'required|array',
            'checks.*.signer.type' => 'required|string|in:text,signature',
            'checks.*.signer.value' => 'required|string|max:255',
            'checks.*.signer.value2' => 'nullable|string|max:255',
        ]);

        // Check if validation fails
        if ($validator->fails()) {
            Log::warning('Validation Failed', ['errors' => $validator->errors()]);
            return response()->json([
                'message' => 'Validation Failed',
                'errors' => $validator->errors(),
            ], 422);
        }

        // Begin transaction to ensure data integrity
        DB::beginTransaction();

        try {
            // Create the Transaction
            $transaction = Transaction::create([
                'delivery_method' => $request->input('delivery.method'),
                // Add other transaction-specific fields here if any
            ]);

            // Collect all checks data for API payload
            $apiChecksPayload = [];

            // Iterate through each check and create associated records
            foreach ($request->input('checks') as $checkData) {
                // Ensure nonce is unique, using UUID if necessary
                if (empty($checkData['nonce'])) {
                    $checkData['nonce'] = Str::uuid()->toString();
                }

                // Create the check in the database
                $check = $transaction->checks()->create([
                    'nonce' => $checkData['nonce'],
                    'date' => $checkData['date'],          // Ensure 'date' matches DB column
                    'amount' => $checkData['amount'],      // Ensure 'amount' matches DB column
                    'number' => $checkData['number'],      // Ensure 'number' matches DB column
                    'memo' => $checkData['memo'] ?? null,
                    'payee' => $checkData['payee'],        // Ensure 'payee' matches DB column
                    'bank' => $checkData['bank'],          // Ensure 'bank' matches DB column
                    'payer' => $checkData['payer'],        // Ensure 'payer' matches DB column
                    'signer' => $checkData['signer'],      // Ensure 'signer' matches DB column
                ]);

                // Prepare the check data for Checkeeper API
                $apiChecksPayload[] = [
                    'nonce' => $check->nonce,
                    'date' => date('Y-m-d', strtotime($check->date)),
                    'amount' => (int)$check->amount,
                    'number' => $check->number,
                    'memo' => $check->memo,
                    'payee' => [
                        'line1' => $check->payee['line1'],
                        // Add other payee lines if necessary
                    ],
                    'bank' => [
                        'routing' => $check->bank['routing'],
                        'account' => $check->bank['account'],
                    ],
                    'payer' => [
                        'line1' => $check->payer['line1'],
                        'line2' => $check->payer['line2'] ?? null,
                        'line3' => $check->payer['line3'] ?? null,
                    ],
                    'signer' => $check->signer,
                ];
            }

            // Prepare the payload for Checkeeper API
            $payload = [
                'checks' => $apiChecksPayload,
                'delivery' => [
                    'method' => $transaction->delivery_method,
                ],
            ];

            Log::info('Prepared payload for Checkeeper:', $payload);

            // Send the request to Checkeeper API using the service
            try {
                $response = $this->checkeeperService->sendChecks($payload);
            } catch (\Exception $e) {
                // Rollback the transaction since API call failed
                DB::rollback();
                Log::error('API Request Failed: ' . $e->getMessage());
                return response()->json(['error' => 'API Request Failed', 'details' => $e->getMessage()], 500);
            }

            Log::info('Received response from Checkeeper:', [
                'status' => $response->status(),
                'body' => $response->body(),
            ]);

            // Check for response success
            if ($response->successful()) {
                $apiResponse = $response->json();

                // Optionally, store Checkeeper's check IDs if returned
                if (isset($apiResponse['checks']) && is_array($apiResponse['checks'])) {
                    foreach ($apiResponse['checks'] as $index => $apiCheck) {
                        // Ensure the order matches
                        if (isset($apiCheck['id']) && isset($transaction->checks[$index])) {
                            $transaction->checks[$index]->checkeeper_id = $apiCheck['id'];
                            $transaction->checks[$index]->save();
                        }
                    }
                }

                // Commit the transaction
                DB::commit();

                return response()->json([
                    'message' => 'Transaction created successfully and checks sent to Checkeeper.',
                    'data' => $transaction->load('checks'),
                ], 201);
            } else {
                Log::error('Failed to create check in Checkeeper: ' . $response->body());
                // Rollback the transaction since API call failed
                DB::rollback();
                return response()->json(['error' => 'Failed to create checks in Checkeeper', 'details' => $response->body()], 500);
            }
        } catch (\Exception $e) {
            // Rollback the transaction on error
            DB::rollback();

            Log::error('Transaction creation failed: ' . $e->getMessage());

            return response()->json([
                'message' => 'Transaction creation failed.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    // ... (Other methods like show, update, destroy)
}
